// Twister include file by Peter Popov
// Generates twisted objects like coils and helices
// swirled around the y axis.
// The twister "grows" in the +y direction (fon anims etc.)

// file version 0.9b

// You need POV-Ray 3.1 for this one

#debug "\nIncluding twister.inc\n\n"
#include "twistini.inc"
#if (twister_direction=0)
	#error "Direction cannot be 0!"
#end

// -------------------------------------------------------------------------
// Short variable names

#declare H = twister_height;

#declare RX  = radius_x;
#declare RXF = radius_x_frequency;
#declare RXP = radius_x_phase*2*pi;
#declare RXR = radius_x_amplitude;

#declare RZ  = radius_z;
#declare RZF = radius_z_frequency;
#declare RZP = radius_z_phase*2*pi;
#declare RZR = radius_z_amplitude;

#declare MR  = element_radius;
#declare MRF = element_radius_frequency;
#declare MRP = element_radius_phase*2*pi;
#declare MRR = element_radius_amplitude;

#declare D   = twister_density;
#declare DF  = twister_density_frequency;
#declare DP  = twister_density_phase*2*pi;
#declare DR  = twister_density_amplitude;

#declare Dir = twister_direction / abs (twister_direction);
#declare S   = twister_accuracy;

#declare TT=0;
#declare TH=0;
#declare TTA=TT/S;

// End of variables list
// -------------------------------------------------------------------------

// -------------------------------------------------------------------------
// Initialization

#declare N = 0;

#declare C  = twister_start_clock;
#declare dC = 1/S;
#declare _C = 2*pi*C;
#declare _dC = 1/S*2*pi;

#include "twistmac.inc"

#declare Last = E_DEFAULT (_C-_dC, Dir, RX, RXR, RXF, RXP, RZ, RZR, RZF, RZP)
                +
                A_DEFAULT (C-dC, H, DR, DF, DP);
                
#declare RLast = R_DEFAULT (_C-_dC, MR, MRR, MRF, MRP);

#declare ZMin=Last.z;
#declare XMin=Last.x;
#declare YMin=Last.y;
#declare ZMax=Last.z;
#declare XMax=Last.x;
#declare YMax=Last.y;
#declare RMax=0;

#declare C  = C+1/S;
#declare _C = 2*pi*C;

#declare Current = E_DEFAULT (_C, Dir, RX, RXR, RXF, RXP, RZ, RZR, RZF, RZP)
                   +
                   A_DEFAULT (C, H, DR, DF, DP);
                   
#declare RCurrent= R_DEFAULT (_C, MR, MRR, MRF, MRP);

#declare Next = E_DEFAULT ( _C+_dC, Dir, RX, RXR, RXF, RXP, RZ, RZR, RZF, RZP)
                   +
                   A_DEFAULT (C+dC, H, DR, DF, DP);

#declare RNext = R_DEFAULT (_C+_dC, MR, MRR, MRF, MRP);

#if ( Current.x > XMax ) #declare XMax=Current.x; #end
#if ( Current.y > YMax ) #declare YMax=Current.y; #end
#if ( Current.z > ZMax ) #declare ZMax=Current.z; #end

#if ( Current.x < XMin ) #declare XMin=Current.x; #end
#if ( Current.y < YMin ) #declare YMin=Current.y; #end
#if ( Current.z < ZMin ) #declare ZMin=Current.z; #end

#if( RCurrent > RMax ) #declare RMax=RCurrent; #end

// End of initialization section
// -------------------------------------------------------------------------

// -------------------------------------------------------------------------
// Twister object declaration

#declare Twister =

union
{
    sphere { Last , RLast texture { twister_texture} }
    #while (C <= twister_end_clock)
        cone
        {
        Last, RLast , Current, RCurrent
        #if (use_uv_mapping)
            texture
            {
                twister_texture
                rotate y*TT
                translate y*TH
                Reorient (Last, Current)
            }
        #end
        }

        intersection
        {
            sphere { Current, RCurrent }
            union
            {
                intersection
                {
                    plane { Next-Current, 0 translate Current }
                    plane { Current-Last, 0 translate Current inverse }
                }
        
                intersection
                {
                    plane { Next-Current, 0 translate Current inverse}
                    plane { Last-Current, 0 translate Current inverse}
                }
            }
            #if (use_uv_mapping)
                texture
                {
                    twister_texture
                    rotate y*TT
                    translate y*TH
                    Reorient (Last, Current)
                }
            #end
            bounded_by { sphere { Current, RCurrent*1.000001 } }
        }

        #declare TT=TT+TTA;
        #declare TH=TH-vlength(Last-Current);
        #declare Last=Current;
        #declare RLast=RCurrent;
        #declare Current=Next;
        #declare RCurrent=RNext;

        #declare C = C + 1/S;
        #declare _C = 2*pi*C;

        #declare Next = E_DEFAULT ( _C+_dC, Dir, RX, RXR, RXF, RXP, RZ, RZR, RZF, RZP )
                                  +
                                  A_DEFAULT (C, H, DR, DF, DP);
                                  
        #declare RNext = R_DEFAULT (_C+_dC, MR, MRR, MRF, MRP);

        #if ( Current.x > XMax ) #declare XMax=Current.x; #end
        #if ( Current.y > YMax ) #declare YMax=Current.y; #end
        #if ( Current.z > ZMax ) #declare ZMax=Current.z; #end

        #if ( Current.x < XMin ) #declare XMin=Current.x; #end
        #if ( Current.y < YMin ) #declare YMin=Current.y; #end
        #if ( Current.z < ZMin ) #declare ZMin=Current.z; #end

        #if ( RCurrent > RMax ) #declare RMax=RCurrent; #end
        
// Comment this out if you don't like it        
        #if (abs(div(C*100,1)-C*100)<0.1)
            #debug concat (str(100*(C-twister_start_clock)/(twister_end_clock-twister_start_clock),3,0),"% complete\r")
        #end

    #end // of main cycle


// Bounding not tested yet... should work but will it be faster? Who knows...
    #if (twister_bounding_type=1)
        bounded_by { box { <XMin,YMin,ZMin>-RMax, <XMax,YMax,ZMax>+RMax scale 1.000001 } }
    #end

    #if (twister_bounding_type=2)
        #if (sqrt(XMax*XMax+ZMax*ZMax)>sqrt(XMin*XMin+ZMin*ZMin))
            bounded_by { cylinder { -RMax*y, (RMax+YMax)*y, RMax+(sqrt(XMax*XMax+ZMax*ZMax)) } }
        #else
            bounded_by { cylinder { -RMin*y, (RMin+YMin)*y, RMin+(sqrt(XMin*XMin+ZMin*ZMin)) } }
        #end
    #end

// This will center the twister if the variation in density is not great.
    translate -y*H/2
}

#debug "\n"

// Hope you find this .inc useful.
// Bug, comments, criticism, suggestions etc. at peter.popov@usa.net
// Make sure the word "twister" is somewhere in the subject line 
// I also hang at news.povray.org or comp.graphics.rendering.raytracing
