// The macros used in twister.inc
// by Peter Popov
// ------------------------------

// file version 0.9b

// this is John VanSickle's Reorient macro

#macro Reorient (Last, Current)
    #local V2 = Current-Last;
    #local V1 = y*vlength(Last-Current);

    #local VX1=vnormalize(V1);
    #local VX2=vnormalize(V2);
    #local VY=vnormalize(vcross(VX2,VX1));
    #local VZ1=vcross(VY,VX1);
    #local VZ2=vcross(VY,VX2);

    matrix < VX1.x, VY.x, VZ1.x,
             VX1.y, VY.y, VZ1.y,
             VX1.z, VY.z, VZ1.z,
             0,     0,      0 >

    matrix < VX2.x, VX2.y, VX2.z,
             VY.x,  VY.y,  VY.z,
             VZ2.x, VZ2.y, VZ2.z,
             0,     0,     0 >

    translate Last
#end

// the function of the twister axis; currently it
// can only be changed if it coincides with y (i.e.
// x and z are 0

#macro A_DEFAULT (C, H, DR, DF, DP)

    <0,C*(H+DR*C*sin(2*pi*C*DF+DP)),0>

#end

// The function that describes twisting
// It can be customised, but y must be 0
// for it to work properly. I will extend
// this capability if I find the nasty bug
// that currently prevents me from doing it

#macro E_DEFAULT( C, Dir,
                  RX, RXR, RXF, RXP,
                  RZ, RZR, RZF, RZP
                )

  <Dir*cos(C*D)*(RX+RXR*sin(C*RXF+RXP)), 
   0,
   Dir*sin(C*D)*(RZ+RZR*sin(C*RZF+RZP))>
#end

// This function describes the thickness
// of the twister at any point along its
// length. It can be customised to whatever
// you wish

#macro R_DEFAULT (C, MR, MRR, MRF, MRP)

    MR+MRR*sin(C*MRF+MRP)
    
#end


/*--------------------------------------------
// The following macros will be used at a later
// stage to allow custom functions to be used
// for the shape of the coil. Currently there
// is a bug I cannot track down that causes the
// return value to be the same. degenerating the
// cylinder. Maybe you can? :)

#macro Vreorient2(axis1, axis2, point)

// thanks to Jerry Anning for this macro

  #local Vx2 = vnormalize (axis2 - axis1);
  #local Vx1 = <0, 1, 0>;
  #local Vy = vnormalize(vcross(Vx2, Vx1));
  #local Vz1 = vnormalize(vcross(Vy, Vx1));
  #local Vz2 = vnormalize(vcross(Vy, Vx2));
  #local A1 = vdot(point, Vx1);
  #local A2 = vdot(point, Vy);
  #local A3 = vdot(point, Vz1);
  #local Apos = <vdot(point, Vx1), vdot(point, Vy), vdot(point, Vz1)>;
  #local B1 = <Vx2.x, Vy.x, Vz2.x>;
  #local B2 = <Vx2.y, Vy.y, Vz2.y>;
  #local B3 = <Vx2.z, Vy.z, Vz2.z>;
  <vdot(Apos, B1), vdot(Apos, B2), vdot(Apos, B3)> + axis1
#end

#macro E_DEFAULT( C, Dir,
                  RX, RXR, RXF, RXP,
                  RZ, RZR, RZF, RZP,
                  H, DR, DF, DP
                )

  #local point = <Dir*cos(C*D)*(RX+RXR*sin(C*RXF+RXP)), 
                 0, 
                 Dir*sin(C*D)*(RZ+RZR*sin(C*RZF+RZP))>;
  Vreorient2 (A_DEFAULT (C, H, DR, DF, DP), 
              A_DEFAULT (C+dC, H, DR, DF, DP), 
              point)
#end

--------------------------------------------*/

// Bug, comments, criticism, suggestions etc. at peter.popov@usa.net
// Make sure the word "twister" is somewhere in the subject line 
// I also hang at news.povray.org or comp.graphics.rendering.raytracing
